local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikItemLevel = BaikCore.BaikItemLevel
local CombatSafe = BaikCore.CombatSafe
local Log = BaikCore.Log
local Hook = BaikCore.Hook

local Badge = BaikItemLevel.Badge

-- Add Module
local Bag = BaikItemLevel:NewModule("Bag", "AceHook-3.0", "AceEvent-3.0")

-- CONSTANTS
local BAGS = nil

-- Private Methods
local function GetBagFrame(containerId)
    local bag = _G[string.format("ContainerFrame%d", containerId + 1)]
    local size = GetContainerNumSlots(containerId)
    local slots = {}
    for idx=1, size, 1 do
        slots[idx] = _G[string.format("ContainerFrame%dItem%d",
                                      containerId + 1, size - idx + 1)]
    end

    return bag, slots
end

local function GetBags()
    if BAGS then
        return BAGS
    end

    local bags = {}
    for idx=0, NUM_BAG_FRAMES, 1 do
        local bagFrame, slots = GetBagFrame(idx)
        bags[bagFrame] = slots
    end
    BAGS = bags

    return bags
end

local function Setup()
    local bags = GetBags()
    for bagFrame, slots in pairs(bags) do
        local function GetItemLink(slot)
            local id = bagFrame:GetID()
            return GetContainerItemLink(id, slot:GetID())
        end
        Hook:EnsureHookScript(Bag, bagFrame, "OnShow", "Load")
        for _, slot in pairs(slots) do
            Badge:Setup(slot, GetItemLink)
        end
    end
end

-- Bag API
function Bag:Load(bagFrame)
    local bags = GetBags()
    local slots = bags[bagFrame]

    for _, slot in pairs(slots) do
        Badge:Update(slot)
    end
end

function Bag:LoadAll()
    local bags = GetBags()
    for bagFrame, slots in pairs(bags) do
        for _, slot in pairs(slots) do
            Badge:Update(slot)
        end
    end
end

-- Ace3 Callbacks
function Bag:OnInitialize()
    Log:v("Bag_OnInitialize")
    CombatSafe:Setup(Setup)
end

function Bag:OnEnable()
    self:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE", "LoadAll")
    self:RegisterEvent("BAG_UPDATE", "LoadAll")
    self:RegisterEvent("BAG_UPDATE_DELAYED", "LoadAll")
    self:RegisterEvent("PLAYERBANKSLOTS_CHANGED", "LoadAll")
    self:RegisterMessage(BaikItemLevel.MESSAGE, "LoadAll")
    Log:v("Bag_OnEnable")
end

function Bag:OnDisable()
    Log:v("Bag_OnDisable")
end
