local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikItemLevel = BaikCore.BaikItemLevel
local CombatSafe = BaikCore.CombatSafe
local Log = BaikCore.Log
local Hook = BaikCore.Hook

local Badge = BaikItemLevel.Badge

-- Add Module
local Bank = BaikItemLevel:NewModule("Bank", "AceHook-3.0", "AceEvent-3.0")

-- CONSTANTS
local FRAMES = nil
local BAG_START = 5
local BAG_END = 12

-- Private Methods
local function IsStale(containerId, slots)
    local isBank = containerId == BANK_CONTAINER
    local size = GetContainerNumSlots(containerId)

    return not isBank and size ~= #slots
end

local function GetBagFrame(containerId)
    local bag = _G[string.format("ContainerFrame%d", containerId + 1)]
    local size = GetContainerNumSlots(containerId)
    local slots = {}
    for idx=1, size, 1 do
        slots[idx] = _G[string.format("ContainerFrame%dItem%d",
                                      containerId + 1, size - idx + 1)]
    end

    return bag, slots
end

local function GetBankFrame()
    local slots = {}
    local size = GetContainerNumSlots(BANK_CONTAINER)

    for idx=1, size, 1 do
        slots[idx] = _G[string.format("BankFrameItem%d", idx)]
    end

    return BankFrame, slots 
end

local function GetFrames()
    if FRAMES then
        return FRAMES
    end

    local frames = {}
    for idx=BAG_START, BAG_END, 1 do
        local bagFrame, slots = GetBagFrame(idx)
        frames[bagFrame] = slots
    end
    local bankFrame, slots = GetBankFrame()
    frames[bankFrame] = slots

    FRAMES = frames

    return frames
end

local function SetupBag(bagFrame, slots)
    local function GetItemLink(slot)
        local id = bagFrame:GetID()
        return GetContainerItemLink(id, slot:GetID())
    end
    Hook:EnsureHookScript(Bank, bagFrame, "OnShow", "Load")
    for _, slot in pairs(slots) do
        Badge:Setup(slot, GetItemLink)
    end
end

local function Setup()
    local bags = GetFrames()
    for bagFrame, slots in pairs(bags) do
        SetupBag(bagFrame, slots)
    end

    Bank:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE", "LoadAll")
    Bank:RegisterEvent("BAG_UPDATE", "LoadAll")
    Bank:RegisterEvent("BAG_UPDATE_DELAYED", "LoadAll")
    Bank:RegisterEvent("PLAYERBANKSLOTS_CHANGED", "LoadAll")
    Bank:RegisterMessage(BaikItemLevel.MESSAGE, "LoadAll")
end

-- Bank API
function Bank:Load(bagFrame)
    local bags = GetFrames()
    local bagId = bagFrame:GetID()
    local slots = bags[bagFrame]

    if IsStale(bagId, slots) then
        _, slots = GetBagFrame(bagId)
        bags[bagFrame] = slots
        SetupBag(bagFrame, slots)
    end

    for _, slot in pairs(slots) do
        Badge:Update(slot)
    end
end

function Bank:LoadAll()
    local bags = GetFrames()
    for bagFrame, slots in pairs(bags) do

        local bagId = bagFrame:GetID()
        if IsStale(bagId, slots) then
            _, slots = GetBagFrame(bagId)
            bags[bagFrame] = slots
            SetupBag(bagFrame, slots)
        end

        for _, slot in pairs(slots) do
            Badge:Update(slot)
        end
    end
end

-- Ace3 Callbacks
function Bank:OnInitialize()
    Log:v("Bank_OnInitialize")
    CombatSafe:Setup(Setup)
end

function Bank:OnEnable()
    Log:v("Bank_OnEnable")
end

function Bank:OnDisable()
    Log:v("Bank_OnDisable")
end
