local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikItemLevel = BaikCore.BaikItemLevel
local CombatSafe = BaikCore.CombatSafe
local Log = BaikCore.Log
local Hook = BaikCore.Hook

local Badge = BaikItemLevel.Badge

-- Add Module
local Character = BaikItemLevel:NewModule("Character", "AceHook-3.0", "AceEvent-3.0")

-- Constants
local FRAMES = nil
local ROOT_FRAME = nil

-- Private Method
local function GetFrames()
    if FRAMES then
        return FRAMES
    end

    local frames = {
        [INVSLOT_HEAD] = CharacterHeadSlot,
        [INVSLOT_NECK] = CharacterNeckSlot,
        [INVSLOT_SHOULDER] = CharacterShoulderSlot,
        [INVSLOT_BODY] = CharacterShirtSlot,
        [INVSLOT_CHEST] = CharacterChestSlot,
        [INVSLOT_WAIST] = CharacterWaistSlot,
        [INVSLOT_LEGS] = CharacterLegsSlot,
        [INVSLOT_FEET] = CharacterFeetSlot,
        [INVSLOT_WRIST] = CharacterWristSlot,
        [INVSLOT_HAND] = CharacterHandsSlot,
        [INVSLOT_FINGER1] = CharacterFinger0Slot,
        [INVSLOT_FINGER2] = CharacterFinger1Slot,
        [INVSLOT_TRINKET1] = CharacterTrinket0Slot,
        [INVSLOT_TRINKET2] = CharacterTrinket1Slot,
        [INVSLOT_BACK] = CharacterBackSlot,
        [INVSLOT_MAINHAND] = CharacterMainHandSlot,
        [INVSLOT_OFFHAND] = CharacterSecondaryHandSlot,
        [INVSLOT_TABARD] = CharacterTabardSlot
    }
    FRAMES = frames

    return frames
end

local function GetRootFrame()
    if ROOT_FRAME then
        return ROOT_FRAME
    end

    local frame = PaperDollFrame
    ROOT_FRAME = frame

    return frame
end

local function Setup()
    local frames = GetFrames()

    local function GetItemLink(slot)
        local link = GetInventoryItemLink("player", slot:GetID())
        return link
    end
    for _, slot in pairs(frames) do
        Badge:Setup(slot, GetItemLink)
    end

    local rootFrame = GetRootFrame()
    Hook:EnsureHookScript(Character, rootFrame, "OnShow", "Load")

    Character:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", "Load")
    Character:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE", "Load")
    Character:RegisterMessage(BaikItemLevel.MESSAGE, "Load")
end

-- Character API
function Character:Load()
    local slots = GetFrames()

    for _, slot in pairs(slots) do
        Badge:Update(slot)
    end
end

-- Ace3 Callbacks
function Character:OnInitialize()
    Log:v("Character_OnInitialize")
end

function Character:OnEnable()
    CombatSafe:Setup(Setup)
    Log:v("Character_OnEnable")
end

function Character:OnDisable()
    Log:v("Character_OnDisable")
end
