local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikItemLevel = BaikCore.BaikItemLevel
local CombatSafe = BaikCore.CombatSafe
local Log = BaikCore.Log
local Hook = BaikCore.Hook

local Badge = BaikItemLevel.Badge

-- Add Module
local Inspect = BaikItemLevel:NewModule("Inspect", "AceHook-3.0", "AceEvent-3.0")

-- Constants
local FRAMES = nil
local ROOT_FRAME = nil

-- Private Method
local function GetFrames()
    if FRAMES then
        return FRAMES
    end

    local frames = {
        [INVSLOT_HEAD] = _G["InspectHeadSlot"],
        [INVSLOT_NECK] = _G["InspectNeckSlot"],
        [INVSLOT_SHOULDER] = _G["InspectShoulderSlot"],
        [INVSLOT_BODY] = _G["InspectShirtSlot"],
        [INVSLOT_CHEST] = _G["InspectChestSlot"],
        [INVSLOT_WAIST] = _G["InspectWaistSlot"],
        [INVSLOT_LEGS] = _G["InspectLegsSlot"],
        [INVSLOT_FEET] = _G["InspectFeetSlot"],
        [INVSLOT_WRIST] = _G["InspectWristSlot"],
        [INVSLOT_HAND] = _G["InspectHandsSlot"],
        [INVSLOT_FINGER1] = _G["InspectFinger0Slot"],
        [INVSLOT_FINGER2] = _G["InspectFinger1Slot"],
        [INVSLOT_TRINKET1] = _G["InspectTrinket0Slot"],
        [INVSLOT_TRINKET2] = _G["InspectTrinket1Slot"],
        [INVSLOT_BACK] = _G["InspectBackSlot"],
        [INVSLOT_MAINHAND] = _G["InspectMainHandSlot"],
        [INVSLOT_OFFHAND] = _G["InspectSecondaryHandSlot"],
        [INVSLOT_TABARD] = _G["InspectTabardSlot"]
    }
    FRAMES = frames

    return frames
end

local function GetRootFrame()
    if ROOT_FRAME then
        return ROOT_FRAME
    end

    local frame = _G["InspectPaperDollItemsFrame"]
    ROOT_FRAME = frame

    return frame
end

local function Setup()
    local frames = GetFrames()

    local rootFrame = GetRootFrame()
    local function GetItemLink(slot)
        local link = GetInventoryItemLink(InspectFrame.unit, slot:GetID())
        return link
    end
    for _, slot in pairs(frames) do
        Badge:Setup(slot, GetItemLink)
    end

    Hook:EnsureHookScript(Inspect, rootFrame, "OnShow", "Load")
    Inspect:RegisterEvent("UNIT_INVENTORY_CHANGED")
    Inspect:RegisterEvent("PLAYER_AVG_ITEM_LEVEL_UPDATE", "Load")
    Inspect:RegisterEvent("INSPECT_READY", "Load")
    Inspect:RegisterMessage(BaikItemLevel.MESSAGE, "Load")
end

-- Inspect API
function Inspect:Load()
    local slots = GetFrames()

    for _, slot in pairs(slots) do
        Badge:Update(slot)
    end
end

function Inspect:ADDON_LOADED()
    if IsAddOnLoaded("Blizzard_InspectUI") then
        Setup()
        self:UnregisterEvent("ADDON_LOADED")
    end
end

function Inspect:UNIT_INVENTORY_CHANGED(self, unit, ...)
    if InspectFrame.unit == unit then
        self:Load()
    end
end

-- Ace3 Callbacks
function Inspect:OnInitialize()
    Log:v("Inspect_OnInitialize")
end

function Inspect:OnEnable()
    if IsAddOnLoaded("Blizzard_InspectUI") then
        CombatSafe:Setup(Setup)
    else
        self:RegisterEvent("ADDON_LOADED")
    end
    Log:v("Inspect_OnEnable")
end

function Inspect:OnDisable()
    Log:v("Inspect_OnDisable")
end
